@@ -1,22 +1,25 @@
 #!/usr/bin/env python3
-import json
 import subprocess
 
+default_version = "0.0.1"
 
 def get_last_version() -> str:
     """Return the version number of the last release."""
-    json_string = (
+    tag_info = (
         subprocess.run(
-            ["gh", "release", "view", "--json", "tagName"],
+            "gh release list | grep -E -v '^.+\\.dev\\d*' | head -n 1",
+            shell=True,
             check=True,
             stdout=subprocess.PIPE,
-            stderr=subprocess.PIPE,
+            stderr=subprocess.PIPE
         )
         .stdout.decode("utf8")
         .strip()
     )
+    tag_fields = tag_info.split('\t')
+    tag = tag_fields[2] if len(tag_fields) > 2 else default_version
 
-    return json.loads(json_string)["tagName"]
+    return tag
 
 
 def bump_patch_number(version_number: str) -> str:
@@ -31,7 +34,9 @@ def create_new_patch_release():
         last_version_number = get_last_version()
     except subprocess.CalledProcessError as err: 
         # The project doesn't have any releases yet.
-        new_version_number = "0.0.1"
+        new_version_number = default_version
+        print(err)
+        print(f'taking default version: {new_version_number}')
     else:
         new_version_number = bump_patch_number(last_version_number)
 
